package calc;


/** The Representation of a Complex Number */

public class Complex implements Comparable {


  static class Debug {
    private static void out(String str) {
      //System.out.println(str);
    }
  }

  public double real = 0,imag = 0;
  
  public final static Complex ZERO = new Complex(0,0);
  public final static Complex ONE = new Complex(1,0);


  public Complex (double real, double imag){

    this.real = real;
    this.imag = imag;
  }

  public Complex (Complex z1){

    this.real = z1.real;
    this.imag = z1.imag;
  }

  public double abs(){
    return  Math.sqrt(real * real + imag * imag);
  }

  public double square(){
    return  real * real + imag * imag;
  }

  public void negate(){
    real = -real;
    imag = - imag;
  }

  public void add(Complex z2){

    real += z2.real;
    imag  += z2.imag;
  }

  public void subtract(Complex z2){

    real -= z2.real;
    imag  -= z2.imag;
  }

  public void mult(Complex z2){
    double newreal = real * z2.real - imag * z2.imag;
    imag = real * z2.imag + imag * z2.real;
    real = newreal;
  }

  public void div(Complex z2){

    double denom = (z2.real * z2.real + z2.imag * z2.imag);
    double newreal = (real * z2.real + imag * z2.imag) / denom;
    imag = (imag * z2.real - real * z2.imag) / denom;
    real = newreal;
  }

  public String toString(){
    return "["+real+" , "+imag+"i]";
  }

  /** we compare the absolute values of complex numbers (needed for sorting) 
      this leads to the inconsistency of ordering with equality (which is normally
      assumed): 
      a.compareTo(b) = 0  =/=>  a.equals(b) !*/
  public int compareTo(Object o){
    if(abs() - ((Complex)o).abs() > 0)
      return 1;
    if(abs() - ((Complex)o).abs() < 0)
      return -1;
    else 
      return 0;
    
  }

  /** two complex numbers are equal, if real- and imaginary part are equal */
  public boolean equals(Object o){
    if(o instanceof Complex && 
       ((Complex)o).real == real && ((Complex)o).imag == imag)
      return true;
    return false;
  }

  /** needed by HashTables (e.g. in ComplexPowerSum.multiply()) */
  public int hashCode(){
    Double realPart = new Double(real);
    Double imagPart = new Double(imag);
    return realPart.hashCode() + imagPart.hashCode();
  }

  /** returns false, if any part of c is NaN (not a number) */
  public boolean isValid(){
    if(Double.isInfinite(real) || Double.isInfinite(real) 
       || Double.isInfinite(real) || Double.isNaN(imag))
      return false;
    return true;
  }

  /* the static methods are used, when a new Object of Type Complex is 
     needed */

  /** the absolute value */
  public static double abs(Complex z1){
    return Math.sqrt(z1.real * z1.real + z1.imag * z1.imag);
			
  }

  /** z -> -z */
  public static Complex negate(Complex z1){
    return new Complex(-z1.real,-z1.imag);
  }

  /** z1, z2 -> z1 + z2 */
  public static Complex add(Complex z1, Complex z2){

    return new Complex(z1.real + z2.real,
		       z1.imag + z2.imag);
  }


  /** z1, z2 -> z1 - z2 */
  public static Complex subtract(Complex z1, Complex z2){
    return new Complex(z1.real - z2.real,
		       z1.imag - z2.imag);
  }

  /** z1, z2 -> z1 * z2 */
  public static Complex mult(Complex z1, Complex z2){

    return new Complex(z1.real * z2.real - z1.imag * z2.imag,
		       z1.real * z2.imag + z1.imag * z2.real);
  }

  /** z1, z2 -> z1 / z2 */
  public static Complex div(Complex z1, Complex z2){

    double denom = (z2.real * z2.real + z2.imag * z2.imag);

    return new Complex((z1.real * z2.real + z1.imag * z2.imag)/ denom,
		       (z1.imag * z2.real - z1.real * z2.imag)/ denom);
  }

  public static Complex sin(Complex z1){
	
    return new Complex(Math.sin(z1.real)/2 * (Math.exp(z1.imag) 
					      + Math.exp(-z1.imag)),
		       Math.cos(z1.real)/2 * (Math.exp(z1.imag) 
					      - Math.exp(-z1.imag)));
  }

  public static Complex cos(Complex z1){
	
    return new Complex(Math.cos(z1.real)/2 * (Math.exp(z1.imag) 
					      + Math.exp(-z1.imag)),
		       Math.sin(z1.real)/2 * (Math.exp(-z1.imag) 
					      - Math.exp(z1.imag)));
  }

  public static Complex exp(Complex z1){
    return new Complex(Math.exp(z1.real)*Math.cos(z1.imag),
		       Math.exp(z1.real)*Math.sin(z1.imag));
  }
	  

  /** the general power function */
  public static Complex pow(Complex z1, Complex z2){
    if(z1.real == 0 && z1.imag == 0)
      return new Complex(0,0);
    if(z1.imag == 0 && z2.imag == 0)
      return new Complex(Math.pow(z1.real,z2.real),0);
    return exp(mult(z2,ln(z1)));
  }


  /** the natural logarithm */
  public static Complex ln(Complex z1){
    return new Complex(Math.log(abs(z1)), Math.atan(z1.imag/z1.real));
  }

  /** for usability, does the same as ln */
  public static Complex log(Complex z1){
    return ln(z1);
  }
	
  /** only for testing purposes */
  public static void main(String args[]){
    if(args.length <4)
      System.out.println("needs 4 args: real1 imag1 real2 imag2");
	
    Complex c1 = new Complex(Double.valueOf(args[0]).doubleValue(),
			     Double.valueOf(args[1]).doubleValue());
    Complex c2 = new Complex(Double.valueOf(args[2]).doubleValue(),
			     Double.valueOf(args[3]).doubleValue());

    System.out.println("c1 = "+c1+"\nc2 = "+c2);

    System.out.println("c1 / c2 = "+ Complex.div(c1,c2));
    System.out.println("c1 * c2 = "+ Complex.mult(c1,c2));

    System.out.println("c1 + c2 = "+Complex.add(c1,c2));
    System.out.println("c1 - c2 = "+Complex.add(c1,Complex.negate(c2)));
    System.out.println("c1 ^ c2 = "+Complex.pow(c1,c2));
    System.out.println("ln c1 = "+Complex.ln(c1));
    System.out.println("exp (ln c1) = "+Complex.exp(ln(c1)));
    System.out.println("exp c1) = "+Complex.exp(c1)); 
    //System.out.println("zeta (c1) = "+Complex.zeta_fastest(c1).abs());
    //System.out.println("zeta (c1) = "+Complex.zeta_fast(c1).abs());
  }

}
