package calc;

import java.util.*;


/** This class takes a string representing an arithmetic expression, a
    pair of ranges and performs the calculations for a row of elements 
    in this range by feeding the operational tree (produced by 
    <code>ExprParser</code>) with the apropriate values 

    @see ExprParser */

public class DoCalc {
    
  /** used to store the representations of variables in the operational 
      tree */

  private Vector variableCalcs = null;

  /** the root node of the operational tree, returned by 
      <code>ExprParse</code> */

  private BaseCalc opRoot = null;


  /** preparses the string, handles it to <code>ExprParser</code>,
      stores the operational tree and	keeps record of the Modules 
      representing the variable t for later insertion (i.e fills
      <code>variableCalc</code> with elements of Type 
      <code>VariableCalc</code> */
    
  public DoCalc(String expr){
	
    String preparsedExpr = preparse(expr);

    try {
      ExprParser parse = new ExprParser(preparsedExpr);
      opRoot = parse.getOperationTree();
      variableCalcs = parse.getVariableCalcs();
    } catch(SyntaxErrorException e) { e.printStackTrace(); }

	
  }


  /** lower cases input and does the following replacements:
      <pre>
      exp -> e
      sin -> s
      cos -> c
      zeta -> 
      re -> r
      im -> m
      </pre>
  */

  private String preparse(String expr){

    expr = expr.toLowerCase();
    expr = replace(expr,"zeta","");
    expr = replace(expr,"exp","e");
    expr = replace(expr,"sin","s");
    expr = replace(expr,"cos","c");
    expr = replace(expr,"re","r");
    expr = replace(expr,"im","m");
	
    System.out.println("preparsed to "+expr);
    return expr;
  }

  /** does the dirty work for preparse() (why is there no regexp-class?!) */

  private String replace(String expr, String what, String by){
    int i = 0;
    i = expr.indexOf(what,i);
    while(expr.regionMatches(i, what, 0, what.length())) {
      System.out.println(i);
    
      expr = expr.substring(0,i) + by 
	+ expr.substring(i + what.length());
      System.out.println(expr);
      i = expr.indexOf(what,i);
    
    }
    return expr;
  }


  /** get the Value at a certain position for this Calculation */

  public Complex getValue(double t){
    opRoot.clear();
    // insert the argument def into the variable-holders
    for(int k = 0; k < variableCalcs.size(); k++)
      ((VariableCalc) variableCalcs.elementAt(k)).setValue(new Complex(t,0));
    return new Complex(opRoot.result());

  }
  
  /** called by user */

  public static void main(String[] args){
    DoCalc Calc = new DoCalc(args[0]);
    double t_1 = Double.valueOf(args[1]).doubleValue();
    System.out.println("OPTree is : "+Calc.opRoot.toString());
    System.out.println(Calc.getValue(t_1));
  }
}


	
