package calc;

/** The Representation of a Function s(t) including its derivations */
public class Function {
  
  public final boolean physics = false;

  public static final int STEPS = 500;
  public static final double DELTA = 0.001;
  public static final int TYPE_S=0;
  public static final int TYPE_V=1;
  public static final int TYPE_A=2;
  
  public double[] v_values=new double[STEPS];
  public double[] a_values=new double[STEPS];
  private double s_min, s_max, v_min, v_max, a_min, a_max, t_0, t_1;

  private DoCalc calc;
  public String term;

  public Function(String term){
    this.term = term;
    calc = new DoCalc(term);
  }


  /** calculates values of a(t), v(t) and the mins and maxes for a given 
      interval */
  public void calculate(double t_0, double t_1){
    this.t_0 = t_0;
    this.t_1 = t_1;

    // initialize values
    s_max = s(t_0);
    s_min = s(t_0);
    v_min = (s(t_0+DELTA)-s(t_0))/DELTA;
    v_max = v_min;
    v_values[0] = v_min;
    a_min = (s(t_0+2*DELTA)-2*s(t_0+DELTA)+s(t_0))/(DELTA*DELTA);
    a_max = a_min;
    a_values[0] = a_min;
    
    for(int i=1;i < STEPS; i++){
      double t = t_0 + ((double)i)*(t_1-t_0)/(double)STEPS;

      if(s(t) > s_max)
	s_max = s(t);
      if(s(t) < s_min)
	s_min = s(t);
      v_values[i] = (s(t+DELTA)-s(t))/DELTA;
           
      a_values[i] = (s(t+2*DELTA)-2*s(t+DELTA)+s(t))/(DELTA*DELTA);
      //System.out.println("a["+i+"] = "+a_values[i]);

      if(v_values[i] > v_max)
	v_max = v_values[i];
      if(v_values[i] < v_min)
	v_min = v_values[i] ;
      if(a_values[i] > a_max)
	a_max = a_values[i];
      if(a_values[i] < a_min)
	a_min = a_values[i];
    }
  }

  public double gett_0(){
    return t_0;
  }

  public double gett_1(){
    return t_1;
  }

  public double s(double t){
    return calc.getValue(t).real;
  }

  /** returns the maximum function value */
  public double s_max(){
    return s_max;
  }

  /** returns the maximum function value */
  public double s_min(){
    return s_min;
  }

  /** returns the maximum function value */
  public double v_max(){
    return v_max;
  }

  /** returns the maximum function value */
  public double v_min(){
    return v_min;
  }
  /** returns the maximum function value */
  public double a_max(){
    return a_max;
  }

  /** returns the maximum function value */
  public double a_min(){
    return a_min;
  }

  /** returns v(t): the velocity for a given time t */
  public double v(double t){
      return  (s(t+DELTA)-s(t))/DELTA;
  }

  /** returns a(t): the acceleration for a given time t */
  public double a(double t){
    // System.out.println("v("+(int)(((t-t_0)*(double)STEPS)/(t_1-t_0))+") = ");
    // System.out.println(+v_values[(int)(((t-t_0)*(double)STEPS)/(t_1-t_0))]);
      return (s(t+2*DELTA)-2*s(t+DELTA)+s(t))/(DELTA*DELTA);

      //return a_values[(int)((t-t_0)/(t_1-t_0)*(double)STEPS)];
  }

  /** a general interface for function (or derivative) values */
  public double eval(double t, int type){
    switch(type){
    case TYPE_S : return s(t);
    case TYPE_V : return v(t);
    case TYPE_A : return a(t);
    }
    return s(t);
  }
    
  /** gives information, which name the y-Variable has (only relevant for physics=true) */
  public String getYVariable(int type){
    if(!physics)
      return "y";
    switch(type){
    case TYPE_S : return "s";
    case TYPE_V : return "v";
    case TYPE_A : return "a";
    }
    return "s";
  }

  public static void main(String[] args){
    Function f = new Function("t^3");
    f.calculate(-10,10);
    for(int i=0; i<STEPS-1; i++){
      double t = ((double)i)/((double)STEPS)*20.0d-10.0d;
      System.out.println("v["+t+"] = "+f.v(t));
      } 
    System.out.println("v(4)="+f.v(4.0));
  }
}
