package show;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import calc.Function;

public class Diagram extends Frame {
    
  public final boolean physics = false;

  static final int STEPS=Function.STEPS;
  
  /** values of animationType */
  static final boolean DERIVE=true;
  static final boolean INTEGRATE=false;
  
  /** The Object containing the function we want to draw */
  Function function;

  /** the sleeping Time: to increase or decrease animation speed */
  protected int sleepRate = 10;

  /** the Set of Canvases */
  protected DiagramCanvas[] canvasSet = {new DiagramCanvas(Function.TYPE_S), new DiagramCanvas(Function.TYPE_V), new DiagramCanvas(Function.TYPE_A)};
  protected int numberOfCanvases = 2;
    
  /** a label showing the complex coordinates of the mouse */
  protected Label coordsLabel = new Label("        Coords       ");

  /** for displaying the sleep rate */
  protected Label sleepRateLabel = new Label("Verzgerung = 10ms");
  
  protected Button playPauseButton = new Button("Play");
  protected Button stopButton = new Button("Stop");
  protected boolean playing;
  protected boolean pause;
  protected byte canvasToBeAnimated = 0;
  protected boolean animationType = DERIVE; 

  /** the Main Panel, containing 3 rows (GridLayout) of equal height */
  protected Panel p = new Panel();

  /** the first row, containing f as a textfield */ 
  protected Panel p1 = new Panel();
  protected Panel p_textfield = new Panel();
  protected TextField t_function = new TextField();
  protected TextField t_t_0 = new TextField();
  protected TextField t_t_1 = new TextField();
  /** the second row containing f' */ 
  protected Panel p2 = new Panel();
  /** the third row containing f'' */ 
  protected Panel p3 = new Panel();

  /** the panel holding coords etc. */
  Panel controlPanel = new Panel();
  
  MenuBar bar = new MenuBar();
  Menu view = new Menu("Ansicht");    
  CheckboxMenuItem f_2_view = new CheckboxMenuItem("f'' anzeigen", false);
  Menu animation = new Menu("Animation");
  CheckboxMenuItem f_0_derive = new CheckboxMenuItem("f ableiten", true);
  CheckboxMenuItem f_1_derive = new CheckboxMenuItem("f' ableiten", false);
  CheckboxMenuItem f_1_integrate = new CheckboxMenuItem("f' integrieren", false);
  CheckboxMenuItem f_2_integrate = new CheckboxMenuItem("f'' integrieren", false);

  public void setSleepRate(int rate){
    sleepRate = rate;
    sleepRateLabel.setText("Verzgerung = "+sleepRate+"ms");
  }

  protected void showCoords(double t, double y,String yVariable){

    // beautify all results by cropping the numbers to 3 digits after '.'
    String outx = Double.toString(t);
    String outy = Double.toString(y);
    if(outx.indexOf(".") + 3 < outx.length()) 
      outx = outx.substring(0, outx.indexOf(".") + 4)+(outx.indexOf("E") > -1 ? outx.substring(outx.indexOf("E")-1,outx.length()) : "");      

    if(outy.indexOf(".") + 3 < outy.length()) 
      outy = outy.substring(0, outy.indexOf(".") + 4)+(outy.indexOf("E") > -1 ? outy.substring(outy.indexOf("E")-1,outy.length()) : "");      

    coordsLabel.setText((physics ? "t = " :"x = ")+outx+";  "+yVariable+" = "+outy+";");
  }


  /** this inner class represents the canvas on which everything is
      drawn. We use physical coordinates (depending on the size of the 
      component) and logical coordinates (depending on the given definition 
      range) which are transformed by methods of the outer class */
  public class DiagramCanvas extends Canvas{

    AnimatorThread aThread;    
    DiagramCanvas derivation,parent;

    /** the range in real direction, needs to be stored for transformation */
    protected double[] timeRange = {0.0d,10.0d};

    /** the range in y direction, may be s, v or a */
    protected double[] yRange = {-10.0d,10.0d};

    /** timeRange[1] - timeRange[0], needed for transformation */
    protected double timeDif = 0;

    /** yRange[1] - yRange[0], needed for transformation */
    protected double yDif = 0;

    /** the type of the function: s(t), v(t) or a(t) */
    protected int type; 
    
    /** the affine transformation applied to the Complex-Data for projecting
	it on the screen, its inverse is used by the MouseCoordinateReader to
	calculate the Complex coordinates from physical screen-coordinates */
    protected AffineTransform transform = new AffineTransform();

    public boolean isDoubleBuffered(){
      return true;
    }

    /** sets the ranges for both time and y direction */
    public void setRange(double t_0, double t_1, double y_0, double y_1){
      timeRange[0] = t_0;
      timeRange[1] = t_1;
      yRange[0] = y_0;
      yRange[1] = y_1;
      timeDif = timeRange[1] - timeRange[0];
      yDif = yRange[1] - yRange[0];
      System.out.println("timeDif="+timeDif+"; yDif="+yDif);
    }

    public void setDerivation(DiagramCanvas derivation){
      this.derivation = derivation;
    }

    public void setParent(DiagramCanvas parent){
      this.parent = parent;
    }

    public DiagramCanvas(int type){
      this.type = type;
      setRange(timeRange[0],timeRange[1],yRange[0],yRange[1]);
      addMouseMotionListener(mouseCoordinateReader);
      addMouseListener(mouseCoordinateReader);
    }

    /** store the mouselistener used by this class, so that the subclass can
	remove it by reference */
    protected MouseCoordinateReader mouseCoordinateReader = new MouseCoordinateReader();

    public Color getColor(){
      if(type == Function.TYPE_S)
	return Color.green;
      if(type == Function.TYPE_V)
	return Color.blue;
      if(type == Function.TYPE_A)
	return Color.red;
      return Color.black;
    }

    Line2D dxLine, dyLine, tangent;
    Image im, im_bg;
    
    /** draws a ellipse walking along the function graph. Also draws a tangent for construction of
	derivation if derive==true. Otherwise displays area between graph and x-axis (for 
	integration)*/
    public void drawAnimation(float tpos, boolean animationType){

      float d_x = (float)getSize().width/(float)(8*100);
      float ypos = (float)function.eval(tpos,type);

      // needed only for derivation:
      float f_t = 0;
      float d_f = 0;
      if(animationType == DERIVE){
	f_t = (float)function.eval(tpos,derivation.type);
	d_f = f_t * d_x;
      }


      Graphics2D g_out = (Graphics2D)getGraphics();
	  
      if(g_out != null){	    
	checkImages();
	Image newImage = createImage(this.getSize().width, this.getSize().height);
	Graphics2D g2 = (Graphics2D)newImage.getGraphics();
	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
			    RenderingHints.VALUE_ANTIALIAS_ON);


	Point2D pos = new Point2D.Float(tpos,ypos);
	try {	  
	  float x_factor = Math.abs((float) transform.createInverse().getScaleX());
	  float y_factor = Math.abs((float) transform.createInverse().getScaleY());
	  float width = x_factor * 6.0f;
	  float height = y_factor * 6.0f;
	  
	  // the transformation takes place after the original image 
	  // (already transformed in paint()) has been drawn as background
	  g2.drawImage(im,0,0,null);

	  // if we want to animate the derivation, draw the tangent with dx and dy 
	  if(animationType == DERIVE){
	    determineTransform(g2);
	    g2.setColor(getColor());
	    dxLine = new Line2D.Float(pos,new Point2D.Float(tpos+d_x,ypos));
	    setXStroke(g2);
	    g2.draw(dxLine);

	    g2.setColor(derivation.getColor());
	    dyLine = new Line2D.Float(tpos+d_x,ypos,tpos+d_x,ypos+d_f);
	    setYStroke(g2);
	    g2.draw(dyLine);

	    g2.setColor(Color.black);
	    tangent = new Line2D.Float(tpos-d_x*2.0f,ypos-d_f*2.0f,tpos+d_x*2.0f,ypos+d_f*2.0f);
	    setFunctionStroke(g2);
	    g2.draw(tangent);

	  } else { // do the part for integration
	    
	    // fill the area between graph and x-Axis by drawing a line directly on im_bg
	    Graphics2D bg = (Graphics2D) im_bg.getGraphics();
	    determineTransform(bg);
	    setFunctionStroke(bg);
	    bg.setColor(parent.getColor());
	    bg.draw(new Line2D.Float(tpos,0.0f,tpos,ypos));
	    // the graph itself should be drawn in the original color
	    Ellipse2D dot = new Ellipse2D.Float();
	    dot.setFrameFromCenter(pos,new Point2D.Float(tpos+width/8,(float)ypos+height/8));
	    bg.setColor(getColor());
	    bg.draw(dot);
	    g2.drawImage(im_bg,0,0,null);
	    determineTransform(g2);
	  }
	  // now paint the moving ellipse on the image
	  g2.setColor(g2.getBackground());
	  Ellipse2D newEll = new Ellipse2D.Float();
          newEll.setFrameFromCenter(pos,new Point2D.Float(tpos+width,(float)ypos+height));
	  g2.setColor(Color.black);
	  g2.fill(newEll);
	  //System.out.println("drawing Ellipse at "+tpos+", "+ypos+", width = "+width+", height="+height+";\nv ="+ypos+" (converted by: "+transform);

	  g_out.drawImage(newImage,0,0,null); //transform.createInverse(),null);
	  if(animationType == DERIVE)
	      derivation.drawChildAnimation(tpos,animationType,y_factor);
	  else
	      parent.drawChildAnimation(tpos,animationType,1);
	} catch(Exception ex){ex.printStackTrace();}
      }
    }

    /** this animation is shown if we have been called by the 
	DiagramCanvas containing the mother function (if we derive) or the derivation (if we 
	integrate) */
    void drawChildAnimation(float tpos, boolean animationType, float y_mother_factor){
      float d_x = (float)getSize().width/(float)(8*100);
      float ypos = (float)function.eval(tpos,type)*d_x;
      Graphics2D g_out = (Graphics2D)getGraphics();
	  
      if(g_out != null){	    
	checkImages();
	Image newImage = createImage(this.getSize().width, this.getSize().height);
	Graphics2D g2 = (Graphics2D)newImage.getGraphics();
	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
			    RenderingHints.VALUE_ANTIALIAS_ON);
	
	Point2D pos = new Point2D.Float(tpos,ypos);
	try {	  
	  float x_factor = Math.abs((float) transform.createInverse().getScaleX());
	  float y_factor = Math.abs((float) transform.createInverse().getScaleY());
	  float width = x_factor * 6.0f;
	  float height = y_factor * 6.0f;
	  
	  //drawBackground(g2);
	  Graphics2D bg = (Graphics2D)im_bg.getGraphics();
	  //g2.setTransform(new AffineTransform());
	  determineTransform(bg);
	  setFunctionStroke(bg);
	  bg.setColor(getColor());
	  Ellipse2D valueEll = new Ellipse2D.Float();
	  if(animationType == DERIVE)
	    valueEll.setFrameFromCenter(tpos,ypos*y_factor/y_mother_factor,tpos+width/4.0f,ypos*y_factor/y_mother_factor+height/4.0f);
	  else
	    valueEll.setFrameFromCenter(tpos,ypos/d_x,tpos+width/4.0f,ypos/d_x+height/4.0f);
	  bg.draw(valueEll);
	  g2.drawImage(im_bg,0,0,null);		  
	  determineTransform(g2);
	  setYStroke(g2);
	  Ellipse2D newEll = new Ellipse2D.Float();
          newEll.setFrameFromCenter(tpos,0,tpos+width,height);
	  g2.setColor(Color.black);
	  g2.fill(newEll);
	  
	  g2.setColor(getColor());
	  //System.out.println("ypos="+ypos*y_factor/y_mother_factor);
	  Line2D yLine = null;
	  if(animationType == DERIVE)
	    yLine = new Line2D.Float(tpos,0.0f,tpos,ypos*y_factor/y_mother_factor);
	  else
	    yLine = new Line2D.Float(tpos,0.0f,tpos,ypos/d_x);
	  //draw the line in the Background-Image
	  g2.draw(yLine);
	} catch(Exception ex){ex.printStackTrace();}
	g_out.drawImage(newImage,0,0,null);
      }            
    }

    /** checks if images used for buffering are present, otherwise create them
     */
    void checkImages(){
      Dimension d = getSize();
      if (im == null || im.getWidth(null) != d.width || im.getHeight(null) != d.height) 
	im = createImage(d.width, d.height);
      if (im_bg == null || im_bg.getWidth(null) != d.width || im_bg.getHeight(null) != d.height) 
	im_bg = createImage(d.width, d.height);
    }                                                
    
    public void paint(Graphics g){
      checkImages();
      Graphics2D g2 = (Graphics2D)im.getGraphics();
      //System.out.println("g2.Background is:"+g2.getBackground());
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
			  RenderingHints.VALUE_ANTIALIAS_ON);

      // set transform by calling Diagram.determineTransform()
      drawBackground(g2);
      determineTransform(g2);
      
      // draw the graph of the function       
      g2.setColor(getColor());
      setFunctionStroke(g2);
      GeneralPath path = new GeneralPath();
      path.moveTo((float)timeRange[0],(float)function.eval(timeRange[0],type));
      // iterate through the function
      for(float t = (float)timeRange[0]; t < timeRange[1]; t += (float)(timeDif/STEPS)){
	//System.out.println("LineTo: "+t+", "+function.eval(t,type));
	path.lineTo(t,(float)function.eval((double)t,type));
      }
      g2.draw(path);
      //drawAnimation(5.0f,-5.0f);
      try{
	g.drawImage(im,0,0,null);
				  //transform.createInverse(),null);
      }catch(Exception e){}
    }


    void drawBackground(Graphics2D g_out){
      checkImages();
      Graphics2D g2 = (Graphics2D) im_bg.getGraphics();
      
      // fill with white Background
      g2.setColor(Color.white);     
      g2.setTransform(new AffineTransform());
      g2.fill(new Rectangle2D.Double(0,0,this.getSize().width,this.getSize().height));

      // draw coordinate axes
      determineTransform(g2);
      g2.setColor(Color.black);
      setXStroke(g2);
      g2.draw(new Line2D.Double(timeRange[0], 0, timeRange[1], 0));
      setYStroke(g2);
      g2.draw(new Line2D.Double(0, yRange[0], 0, yRange[1]));
      g_out.drawImage(im_bg,0,0,null);
    }

    void setXStroke(Graphics2D g2){
      g2.setStroke(new BasicStroke((float) (4.0 * yDif / (float)this.getSize().height),
				   BasicStroke.CAP_BUTT, 
				   BasicStroke.JOIN_MITER));
      //System.out.println("Setting Stroke to: "+(2.0 * yDif / (float)this.getSize().height));

    }

    void setYStroke(Graphics2D g2){
 g2.setStroke(new BasicStroke((float) (4.0 * timeDif / (float)this.getSize().width),
				   BasicStroke.CAP_BUTT, 
				   BasicStroke.JOIN_MITER));
      //System.out.println("Setting Stroke to: "+(4.0 * timeDif / (float)this.getSize().width));
    }

    void setFunctionStroke(Graphics2D g2){
      //System.out.println("Setting Stroke to ["+(float) (Math.min(6.0 * yDif / (float)this.getSize().height,timeDif / (float)this.getSize().width))+"]");
      g2.setStroke(new BasicStroke((float) (Math.min(6.0 * yDif / (float)this.getSize().height,6.0 * timeDif / (float)this.getSize().width)),
				   BasicStroke.CAP_SQUARE, 
				   BasicStroke.JOIN_ROUND));
    }
   
    /** returns a double coord with points which have been transformed
	from integer-physical graphics coordinates to double logical space
	coordinates  using the inverse Transformation of the one used in
	paint() */
    public double[] transformXY(int t, int y){
      double[] in = {t,y};
      double[] out = new double[2];
      try {
	transform.createInverse().transform(in,0,out,0,1);
      } catch(Exception ex){ex.printStackTrace();}
      return out;
    }


    /* starts the animation */
    public void startAnimation(boolean anType){
      aThread = new AnimatorThread((float)timeRange[0],(float)(timeDif/(double)STEPS),anType);
      aThread.start();
    }

    /* An inner Class for the Animation */
    private class AnimatorThread extends Thread {

      /* the position of the sprite in logical coords */ 
      float tpos, ypos;
      /* the width of a step in logical coords */
      float tstep;
      boolean anType;
      
      public AnimatorThread(float tpos, float tstep, boolean anType){
	this.tpos = tpos;
	this.ypos = (float)function.eval(tpos,type);
	this.tstep = tstep;
	this.anType = anType;
      }

      public void run(){
	while(playing){
	  try{
	    //System.out.println("Sleeping...");
	    sleep(sleepRate);	   
	  } catch(InterruptedException e){}
	  if(!pause){
	    tpos += tstep;
	    if(tpos > timeRange[1]){
	      repaint();
	      tpos = (float)timeRange[0]; 
	    }
	    // draw the circle at the current position
	    drawAnimation(tpos, anType);
	  }
	}
	    
      }
    }

	
    /** sets the affine transformation, determined by window-size and readDif,
	imagDif */
    public void determineTransform(Graphics2D g2){
      // scale the Image to the appropriate size
      double factorx = ((double) this.getSize().width) / timeDif;
      double factory = ((double) this.getSize().height) / yDif;
    
      transform = new AffineTransform(0.8d * factorx, 0,
				      0, -0.8d * factory,
				      -timeRange[0] * factorx, 
				      Math.max(yRange[1] *0.9d,0.1d*yDif) * factory);
      g2.setTransform(transform);	

    }
  

    /** an inner class, which reads the coordinates of the mouse and transforms
	them to complex coordinates. Also used for determining the gradient
	within a given radius */
  
    public class MouseCoordinateReader extends MouseAdapter implements MouseMotionListener {
    
      public void mouseDragged(MouseEvent e){
	mouseMoved(e);
      }
    
      public void mouseMoved(MouseEvent e){

	// read out mouse-coordinates (modified by offsets) and apply inverse 
	// transformation on them
	double[] coords = transformXY(e.getX(), e.getY());
	showCoords(coords[0],coords[1],function.getYVariable(type));
      }
    }
 
  } // end class Definition DiagramCanvas
      

  public void setFunction(calc.Function f, double t_0, double t_1){
    function=f;
    f.calculate(t_0,t_1);
    System.out.println("a_min="+f.a_min()+"; a_max="+f.a_max());
    canvasSet[0].setRange(t_0,t_1,Math.min(f.s_min()*1.0,-0.2d),Math.max(f.s_max()*1.0,0.2));
    canvasSet[1].setRange(t_0,t_1,Math.min(f.v_min()*1.0,-0.2d),Math.max(f.v_max()*1.0,0.2));
    canvasSet[2].setRange(t_0,t_1,Math.min(f.a_min()*1.0,-0.2d),Math.max(f.a_max()*1.0,0.2));
    canvasSet[0].setDerivation(canvasSet[1]);
    canvasSet[1].setDerivation(canvasSet[2]);
    canvasSet[1].setParent(canvasSet[0]);
    canvasSet[2].setParent(canvasSet[1]);
  }

  public Diagram(calc.Function f, double t_0, double t_1){

    setFunction(f,t_0,t_1);
    view.add(f_2_view);
    f_2_view.addItemListener(new ItemListener() { 
	public void itemStateChanged(ItemEvent e){
	  if(e.getStateChange() == ItemEvent.DESELECTED)
	    numberOfCanvases = 2;
	  else
	    numberOfCanvases = 3;
	  build_p();
	}});
    f_0_derive.addItemListener(new ItemListener() { 
	public void itemStateChanged(ItemEvent e){
	  canvasToBeAnimated = 0;
	  animationType = DERIVE;
	  f_1_derive.setState(false);
	  f_1_integrate.setState(false);
	  f_2_integrate.setState(false);
	  System.out.println("Setting canvasToBeAnimated to "+canvasToBeAnimated);
	  validate();
	  repaint();
	}});
    f_1_derive.addItemListener(new ItemListener() { 
	public void itemStateChanged(ItemEvent e){
	  canvasToBeAnimated = 1;
	  animationType = DERIVE;
	  f_0_derive.setState(false);
	  f_1_integrate.setState(false);
	  f_2_integrate.setState(false);
	  System.out.println("Setting canvasToBeAnimated to "+canvasToBeAnimated);
	  validate();
	  repaint();
	}});
    f_1_integrate.addItemListener(new ItemListener() { 
	public void itemStateChanged(ItemEvent e){
	  canvasToBeAnimated = 1;
	  animationType = INTEGRATE;
	  f_0_derive.setState(false);
	  f_1_derive.setState(false);
	  f_2_integrate.setState(false);
	  System.out.println("Setting canvasToBeAnimated to "+canvasToBeAnimated);
	  validate();
	  repaint();
	}});
    f_2_integrate.addItemListener(new ItemListener() { 
	public void itemStateChanged(ItemEvent e){
	  canvasToBeAnimated = 2;
	  animationType = INTEGRATE;
	  f_0_derive.setState(false);
	  f_1_derive.setState(false);
	  f_1_integrate.setState(false);
	  System.out.println("Setting canvasToBeAnimated to "+canvasToBeAnimated);
	  validate();
	  repaint();
	}});
    animation.add(f_0_derive);
    animation.add(f_1_derive);
    animation.add(f_1_integrate);
    animation.add(f_2_integrate);
    bar.add(view);
    bar.add(animation);
    setMenuBar(bar);
    p1.setLayout(new BorderLayout());
    p1.add(canvasSet[0],BorderLayout.CENTER);
    t_function.setText(function.term);
    t_t_0.setText(""+function.gett_0());
    t_t_1.setText(""+function.gett_1());
    ActionListener al = new ActionListener() {
	public void actionPerformed(ActionEvent e){
	  setFunction(new Function(t_function.getText()),Double.valueOf(t_t_0.getText()).doubleValue(), Double.valueOf(t_t_1.getText()).doubleValue());
	  build_p();
	}};
    t_function.addActionListener(al);
    t_t_0.addActionListener(al);
    t_t_1.addActionListener(al);
    p_textfield.add(new Label((physics ? "s(t) = " : "f(x) = ")));
    p_textfield.add(t_function);
    p_textfield.add(new Label(" from "));
    p_textfield.add(t_t_0);
    p_textfield.add(new Label(" to "));
    p_textfield.add(t_t_1);
    p1.add(p_textfield, BorderLayout.NORTH);
    p2.setLayout(new BorderLayout());
    p2.add(canvasSet[1],BorderLayout.CENTER);
    p2.add(new Label((physics ? "v(t)" : "f'(x)")),BorderLayout.NORTH);
    p3.setLayout(new BorderLayout());
    p3.add(canvasSet[2],BorderLayout.CENTER);
    p3.add(new Label((physics ? "a(t)" : "f''(x)")),BorderLayout.NORTH);
    build_p();
    setLayout(new BorderLayout());
    add(p, BorderLayout.CENTER);
    controlPanel.add(coordsLabel);
    controlPanel.add(sleepRateLabel);
    controlPanel.add(playPauseButton);
    controlPanel.add(stopButton);
    playPauseButton.addActionListener(new ActionListener(){
	public void actionPerformed(ActionEvent e){
	  if(playing){
	    if(pause)
	      contAnimation();
	    else
	      pauseAnimation();
	  }
	  else
	    startAnimation();
	}});
    stopButton.addActionListener(new ActionListener(){
	public void actionPerformed(ActionEvent e){
	  if(playing)
	    stopAnimation();
	}});
	  
    add(controlPanel, BorderLayout.SOUTH);
  }
  
  protected void build_p(){
    if(numberOfCanvases == 2)
      build_p_2();
    else build_p_3();
  }

  protected void build_p_3(){
    p.removeAll();
    p.setLayout(new GridLayout(3,1));
    p.add(p1);
    p.add(p2);
    p.add(p3);
    p.validate();
}

  protected void build_p_2(){
    p.removeAll();
    p.setLayout(new GridLayout(2,1));
    p.add(p1);
    p.add(p2);
    validate();
  }


  public void startAnimation(){
    canvasSet[0].repaint();
    canvasSet[1].repaint();
    canvasSet[2].repaint();
    playing = true;
    pause = false;
    playPauseButton.setLabel("Pause");
    canvasSet[canvasToBeAnimated].startAnimation(animationType);
  }

  public void pauseAnimation(){
    pause = true;
    playPauseButton.setLabel("Play");
  }

  public void contAnimation(){
    pause = false;
    playPauseButton.setLabel("Pause");
  }

  public void stopAnimation(){
    playing = false;
    pause = false;
    playPauseButton.setLabel("Play");
    validate();
    repaint();
  }
  
  public static void main(String[] args){
    Diagram d = new Diagram(new Function(args[0]),Double.valueOf(args[1]).doubleValue(),Double.valueOf(args[2]).doubleValue());
    if(args.length > 3)
      d.setSleepRate(Integer.valueOf(args[3]).intValue());
    d.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e){
	  System.exit(0);
	}
      });
    d.setVisible(true);
    d.setSize(400,800);
    d.validate();
    //d.startAnimation();
  }                     
}	
