package show;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import calc.Complex;

public class Diagram extends Container {

  /** the range in real direction, needs to be stored for transformation */
  protected double[] timeRange = {0.0d,10.0d};

  /** the range in y direction, may be s, v or a */
  protected double[] yRange = {-10.0d,10.0d};

  /** a label showing the complex coordinates of the mouse*/
  protected Label showCoords = new Label("Coords");

  /** timeRange[1] - timeRange[0], needed for transformation */
  protected double timeDif = 0;

  /** yRange[1] - yRange[0], needed for transformation */
  protected double yDif = 0;

  /** the affine transformation applied to the Complex-Data for projecting
      it on the screen, its inverse is used by the MouseCoordinateReader to
      calculate the Complex coordinates from physical screen-coordinates */
  protected AffineTransform transform = new AffineTransform();

  /** store the mouselistener used by this class, so that the subclass can
      remove it by reference */
  protected MouseCoordinateReader mouseCoordinateReader = new MouseCoordinateReader();

  /** stores the calculator frontend to compute function values independent
      of the value-grid */
  protected calc.Calculator calculator;

  /** sets the calculator frontend */
  public void setCalculator(calc.Calculator calculator){
    this.calculator = calculator;
  }

  /** returns a Complex number with points which have been transformed
      from integer-physical graphics coordinates to double logical space
      coordinates  using the inverse Transformation of the one used in
      paint() */
  public double[] transformXY(int t, int y){
    double[] in = {t,y};
    double[] out = new double[2];
    try {
      transform.createInverse().transform(in,0,out,0,1);
    } catch(Exception ex){ex.printStackTrace();}
    return out[];
  }


  /** sets the affine transformation, determined by window-size and readDif,
      imagDif */
  public void determineTransform(int width, int height){
    // scale the Image to the appropriate size
    double factorx = ((double) width) / timeDif;
    double factory = ((double) height) / yDif;
    
    transform = new AffineTransform(factorx, 0,
			       0, - factory,
			       - timeRange[0] * factorx, 
			       yRange[1] * factory);
}
  
  /** sets the ranges for both time and y direction */
  public void setRange(double[] timeRange, double[] yRange){
    this.timeRange[0] = timeRange[0];
    this.timeRange[1] = timeRange[1];
    this.yRange[0] = yRange[0];
    this.yRange[1] = yRange[1];
    timeDif = timeRange[1] - timeRange[0];
    yDif = yRange[1] - yRange[0];
  }

  /** an inner class, which reads the coordinates of the mouse and transforms
      them to complex coordinates. Also used for determining the gradient
      within a given radius */
  
  public class MouseCoordinateReader extends MouseAdapter implements MouseMotionListener {
    
    public void mouseMoved(MouseEvent e){

      // read out mouse-coordinates (modified by offsets) and apply inverse 
      // transformation on them
      double[] coords = transformXY(e.getX(), e.getY());

      double t = coords[0];
      double y = coords[1];

      // if numbers are of form xxxE-xx (i.e. too low), they must be set to 
      // zero for displaying purposes
      if(Math.abs(c.real) < 0.001d)
	c.real = 0.0d;
      if(Math.abs(c.imag) < 0.001d)
	c.imag = 0.0d;
      if(Math.abs(f_x) < 0.001d)
	f_x = 0.0d;
      if(Math.abs(f_y) < 0.001d)
	f_y = 0.0d;	

      // here we also have to add some Code here for numbers of form xxxExx 
      // (too high)

      // beautify all results by cropping the numbers to 3 digits after '.'
      String outx = Double.toString(t);
      String outy = Double.toString(y);
      if(outx.indexOf(".") + 4 < outx.length()) 
	outx = outx.substring(0, outx.indexOf(".") + 4);      
      if(outy.indexOf(".") + 4 < outy.length()) 
	outy = outy.substring(0, outy.indexOf(".") + 4);

      showCoords.setText("t = "+outx+"; y = "+outy+";");
    }
    

  /* canvas is handled by child class. In order to get to it from this class,
     this method must be implemented by child */
  public abstract Canvas getCanvas();

  public Field2DDisplay(){
    
    // get Canvas from subclass, initializing it by this
    c = getCanvas();

    //to read out Complex Coordinates
    c.addMouseMotionListener(mouseCoordinateReader);
    c.addMouseListener(mouseCoordinateReader);
    setLayout(new BorderLayout());
    add(c, BorderLayout.CENTER);
    add(showCoords, BorderLayout.SOUTH);
  }
  
}	
